package jp.flashcast.translator.android;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;

import jp.flashcast.translator.android.dao.TranslateDao;
import jp.flashcast.translator.android.model.LanguageModel;
import jp.flashcast.translator.android.model.TranslateModel;
import jp.flashcast.translator.android.model.TweetModel;
import jp.flashcast.translator.android.model.XAuthModel;
import jp.flashcast.translator.android.service.TranslateService;
import jp.flashcast.translator.android.thread.TranslateThread;
import jp.flashcast.translator.android.thread.TweetThread;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.ComponentName;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.ServiceConnection;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.speech.RecognizerIntent;
import android.speech.tts.TextToSpeech;
import android.speech.tts.TextToSpeech.OnInitListener;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.Toast;
import android.widget.ToggleButton;
import net.gimite.jatts.JapaneseTextToSpeech;

public class TranslatorSample11 extends Activity implements Runnable, OnInitListener {
    private ProgressDialog dialog;
    private TranslateModel gmodel;
    private TweetModel tmodel;
    private XAuthModel xmodel;
    private boolean translating;
    private boolean tweeting;
    private TranslateDao dao;
    private ArrayAdapter<LanguageModel> languages;
    private String extraLanguage;
    private static final int VOICE_RECOGNITION_REQUEST_CODE = 9999;
    private static final int TTS_DATA_CHECK_REQUEST_CODE = 9998;
    private boolean recognizable;
	private boolean playable;
    private boolean playable_ja;
    private TextToSpeech tts;
    private JapaneseTextToSpeech jatts;
	@SuppressWarnings("unused")
	private TranslateService service;
    private ServiceConnection connection;
    private boolean binded;
    
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        
        initSpinners();
        initButtons();
        
    	connection = new ServiceConnection() {
    		
    		public void onServiceConnected(ComponentName name, IBinder binder) {
    			service = ((TranslateService.TranslateBinder)binder).getService();
    		}
    		
    		public void onServiceDisconnected(ComponentName name) {
    			service = null;
    		}
    		
    	};
    	
        Intent intent = new Intent(this, TranslateService.class);
        startService(intent);
        bindService(intent, connection, Context.BIND_AUTO_CREATE);
        binded = true;
        
        if (dao == null) {
        	dao = new TranslateDao(this);
        }
    }
    
    private void initSpinners() {
    	Spinner spSrc = (Spinner)findViewById(R.id.srclanguage);
    	Spinner spDst = (Spinner)findViewById(R.id.dstlanguage);
    	
    	String[] labels = getResources().getStringArray(R.array.language_label);
    	String[] values = getResources().getStringArray(R.array.language_value);
    	languages = new ArrayAdapter<LanguageModel>(this, android.R.layout.simple_spinner_item);
    	languages.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
    	
    	for (int i = 0; i < labels.length; i++) {
    		languages.add(new LanguageModel(labels[i], values[i]));
    	}
    	
    	spSrc.setAdapter(languages);
    	spDst.setAdapter(languages);
    }
    
    private void initButtons() {
        // 翻訳方向ボタン
        ToggleButton btnVector = (ToggleButton)findViewById(R.id.vector);
        btnVector.setTextOff(">>");
        btnVector.setTextOn("<<");
        btnVector.setBackgroundResource(android.R.drawable.btn_default);
        
    	// 音声認識の初期化
    	try {
    		PackageManager manager = getPackageManager();
    		List<ResolveInfo> activities = manager.queryIntentActivities(
    			new Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH), 0);
    		
        	if (activities.size() != 0) {
        		recognizable = true;
    		} else {
    			recognizable = false;
    		}
    	}
    	catch (Exception ex) {
    		recognizable = false;
    	}
    	
    	// TextToSpeechの初期化
    	try {
    		Intent intent = new Intent();
    		intent.setAction(TextToSpeech.Engine.ACTION_CHECK_TTS_DATA);
    		startActivityForResult(intent, TTS_DATA_CHECK_REQUEST_CODE);
    	}
		catch (Exception ex) {
			playable = false;
		}
		
    	if (jatts == null) {
        	try {
        		jatts = new JapaneseTextToSpeech(getApplicationContext(), this);
         	   	playable_ja = true;
    		}
    		catch (Exception ex) {
         	   	playable_ja = false;
    		}
    	}
		
        // Voiceボタン
        Button btnSpeak = (Button)findViewById(R.id.voice);
        btnSpeak.setEnabled(recognizable);
        
        if (recognizable) {
	        btnSpeak.setOnClickListener(new OnClickListener() {
	
				public void onClick(View v) {
    				final AlertDialog.Builder builder = new AlertDialog.Builder(TranslatorSample11.this);
    		    	final CharSequence[] labels = getResources().getStringArray(R.array.language_label);
    				
    				builder.setTitle("選択してください。");
    				builder.setItems(labels, new DialogInterface.OnClickListener() {
    		    		
						public void onClick(DialogInterface dialog, int which) {
							showVoiceRecognitionDialog(languages.getItem(which));
						}
    		    	});
    				
    				builder.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {

						public void onClick(DialogInterface dialog, int which) {
							return;
						}
    					
    				});
    				
    				builder.setCancelable(true);
    				builder.create().show();
				}
	        	
	        });
        }
        
        // Playボタン
        Button btnPlay = (Button)findViewById(R.id.play);
        btnPlay.setOnClickListener(new OnClickListener() {

			public void onClick(View v) {
				if (gmodel != null) {
    				if (!gmodel.getTranslated().equals("")) {
    					if (tts.isSpeaking()) {
    						tts.stop();
    					}
    					else if (jatts.isSpeaking()) {
    						jatts.stop();
    					}
    					
    					if (gmodel.getDstLanguage().equals("ja")) {
    						HashMap<String, String> params = new HashMap<String, String>();
    						params.put(JapaneseTextToSpeech.KEY_PARAM_SPEAKER, "female01");
	    					jatts.speak(gmodel.getTranslated(), TextToSpeech.QUEUE_FLUSH, params);
    					}
    					else {
        					tts.setLanguage(getLocale(gmodel.getSrcLanguage()));
        					tts.speak(gmodel.getTranslated(), TextToSpeech.QUEUE_FLUSH, null);
    					}
    				}
				}
			}
        	
        });
        
        // Tweetボタン
        Button btnTweet = (Button)findViewById(R.id.tweet);
        btnTweet.setOnClickListener(new OnClickListener() {

			public void onClick(View v) {
				if (!gmodel.getTranslated().equals("")) {
					tweeting = true;
					
					if (tmodel == null) {
						tmodel = new TweetModel();
					}
					if (xmodel == null) {
						xmodel = new XAuthModel();
					}
					tmodel.setStatus(gmodel.getTranslated());
					
					Tweet();
				}
			}
        	
        });
        
        // 翻訳ボタン
        Button btnTranslate = (Button)findViewById(R.id.translate);
        btnTranslate.setOnClickListener(new OnClickListener() {

			public void onClick(View v) {
    			EditText text;
    			Spinner spSrc;
    			Spinner spDst;
    			
    			ToggleButton btnVector = (ToggleButton)findViewById(R.id.vector);
    			
    			if (btnVector.isChecked()) {
    				text = (EditText)findViewById(R.id.dsttext);
    				spSrc = (Spinner)findViewById(R.id.dstlanguage);
    				spDst = (Spinner)findViewById(R.id.srclanguage);
    			}
    			else {
    				text = (EditText)findViewById(R.id.srctext);
    				spSrc = (Spinner)findViewById(R.id.srclanguage);
    				spDst = (Spinner)findViewById(R.id.dstlanguage);
    			}
    			
    			if (!text.getText().toString().equals("")) {
    				translating = true;
    				
    				if (gmodel == null) {
    					gmodel = new TranslateModel();
    				}
    				
    				gmodel.setOriginal(text.getText().toString());
    				gmodel.setSrcLanguage(((LanguageModel)spSrc.getSelectedItem()).getValue());
    				gmodel.setDstLanguage(((LanguageModel)spDst.getSelectedItem()).getValue());
    				
    				Translate();
    			}
			}
        	
        });
    }
    
    private void setDialog(String message) {
    	if (dialog == null) {
    		dialog = new ProgressDialog(TranslatorSample11.this);
    		dialog.setIndeterminate(true);
    	}
		dialog.setMessage(message);
    	dialog.show();
    }
    
    private void showVoiceRecognitionDialog(LanguageModel locale) {
    	extraLanguage = getLocale(locale);
    	
        Intent intent = new Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH);
        intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL,
                RecognizerIntent.LANGUAGE_MODEL_FREE_FORM);
        intent.putExtra(RecognizerIntent.EXTRA_PROMPT, "TranslateSample8");
        intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, extraLanguage);
        startActivityForResult(intent, VOICE_RECOGNITION_REQUEST_CODE);
    }
    
    private void Translate() {
    	setDialog("Translating... Please wait.");
    	
    	(new TranslateThread(new Handler(), this, gmodel)).start();
    }
    
    private void Tweet() {
    	setDialog("Tweeting... Please wait.");
    	
    	(new TweetThread(new Handler(), this, tmodel, xmodel)).start();
    }
    
	public void run() {
		dialog.dismiss();
		
		if (translating) {
			if (gmodel.isSuccess()) {
				dao.insert(gmodel);
				
				((EditText)findViewById(
					(!((ToggleButton)findViewById(R.id.vector)).isChecked()) ? R.id.dsttext : R.id.srctext))
					.setText(gmodel.getTranslated());
				
				// 音声認識ボタン
				if (recognizable) {
					Button btnRec = (Button)findViewById(R.id.voice);
					btnRec.setEnabled(true);
				}
				
				// 翻訳結果再生ボタン
				Button btnPlay = (Button)findViewById(R.id.play);
				
				if (gmodel.getDstLanguage().equals("ja")) {
					btnPlay.setEnabled(playable_ja);
				}
				else {
					if (playable) {
						Locale locale = getLocale(gmodel.getDstLanguage());
						
						if (tts.isLanguageAvailable(locale) == TextToSpeech.LANG_AVAILABLE) {
							btnPlay.setEnabled(true);
						}
						else {
							btnPlay.setEnabled(false);
						}
					}
				}
			}
			else {
				Toast.makeText(this, "Failed to translate...", Toast.LENGTH_SHORT).show();
			}
			
			translating = false;
		}
		else if (tweeting) {
			if (tmodel.isSuccess()) {
				AlertDialog.Builder builder = new AlertDialog.Builder(this);
				builder.setMessage("Tweet complete.");
				builder.setPositiveButton("OK", new DialogInterface.OnClickListener() {
					
					public void onClick(DialogInterface dialog, int which) {
						return;
					}
					
				});
				
				builder.setCancelable(true);
				builder.create().show();
			}
			else {
				Toast.makeText(this, "Failed to tweet...", Toast.LENGTH_SHORT).show();
			}
			
			tweeting = false;
		}
	}
	
    private String getLocale(LanguageModel model) {
    	String locale;
    	
    	if (model.getValue().equals("en")) {
    		locale = Locale.ENGLISH.toString();
    	}
    	else if (model.getValue().equals("ja")) {
    		locale = Locale.JAPAN.toString();
    	}
    	else if (model.getValue().equals("zh")) {
    		locale = Locale.CHINA.toString();
    	}
    	else if (model.getValue().equals("it")) {
    		locale = Locale.ITALY.toString();
    	}
    	else if (model.getValue().equals("es")) {
    		locale = new Locale("spa", "ESP").toString();
    	}
    	else if (model.getValue().equals("fr")) {
    		locale = Locale.FRANCE.toString();
    	}
    	else if (model.getValue().equals("de")) {
    		locale = Locale.GERMAN.toString();
    	}
    	else if (model.getValue().equals("ko")) {
    		locale = Locale.KOREA.toString();
    	}
    	else if (model.getValue().equals("ru")) {
    		locale = new Locale("ru", "RUS").toString();
    	}
    	else {
    		locale = new Locale("OTHER").toString();
    	}
    	
    	return locale;
    }
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (requestCode == VOICE_RECOGNITION_REQUEST_CODE) {
        	// 音声認識初期化の結果
    		if (resultCode == RESULT_OK) {
    			ArrayList<String> matches = data.getStringArrayListExtra(
    				RecognizerIntent.EXTRA_RESULTS);
    			
    			StringBuilder sb = new StringBuilder();
    			
    			for (int i = 0; i < matches.size(); i++) {
    				sb.append(matches.get(i));
    			}
    			
    			ToggleButton vector = (ToggleButton)findViewById(R.id.vector);
    			
    			if (vector.isChecked()) {
    				((EditText)findViewById(R.id.dsttext)).setText(sb.toString());
    			}
    			else {
    				((EditText)findViewById(R.id.srctext)).setText(sb.toString()); 
    			}
    		}
        }
        else if (requestCode == TTS_DATA_CHECK_REQUEST_CODE) {
        	// TextToSpeech初期化の結果
    		if (resultCode == TextToSpeech.Engine.CHECK_VOICE_DATA_PASS) {
    			// TextToSpeechが利用できる場合
    			if (tts == null) {
    				try {
    					tts = new TextToSpeech(getApplicationContext(), this);
    					playable = true;
    				}
    				catch (Exception ex) {
    					playable = false;
    				}
    			}
    		}
    		else {
    			// TextToSpeechのリソースがインストールされていない場合
    			try {
	    			Intent intent = new Intent();
	    			intent.setAction(TextToSpeech.Engine.ACTION_INSTALL_TTS_DATA);
	    			startActivity(intent);
    			}
    			catch (Exception ex) {
    				playable = false;
    			}
    		}
        }
    	
        super.onActivityResult(requestCode, resultCode, data);
    }
    
    private Locale getLocale(String value) {
    	Locale locale;
    	
    	if (value.equals("en")) {
    		locale = Locale.ENGLISH;
    	}
    	else if (value.equals("ja")) {
    		locale = Locale.JAPANESE;
    	}
    	else if (value.equals("zh")) {
    		locale = Locale.CHINESE;
    	}
    	else if (value.equals("it")) {
    		locale = Locale.ITALIAN;
    	}
    	else if (value.equals("es")) {
    		locale = new Locale("spa");
    	}
    	else if (value.equals("fr")) {
    		locale = Locale.FRENCH;
    	}
    	else if (value.equals("de")) {
    		locale = Locale.GERMAN;
    	}
    	else if (value.equals("ko")) {
    		locale = Locale.KOREAN;
    	}
    	else if (value.equals("ru")) {
    		locale = new Locale("ru");
    	}
    	else {
    		locale = new Locale("OTHER");
    	}
    	
    	return locale;
    }
    
	public void onInit(int status) {
    	Button btnPlay = (Button)findViewById(R.id.play);
    	
    	if (status == TextToSpeech.SUCCESS) {
    		btnPlay.setEnabled(true);
    	}
    	else {
    		btnPlay.setEnabled(false);
    	}
    	
    	if (gmodel == null) {
    		btnPlay.setEnabled(false);
    	}
    	else {
    		btnPlay.setEnabled(!gmodel.getTranslated().equals(""));
    	}
	}
	
    @Override
    protected void onDestroy() {
    	super.onDestroy();
    	
    	if (tts != null) {
    		tts.shutdown();
    	}
    	
    	if (binded) {
    		unbindService(connection);
    	}
    }
}